#include <stdio.h>
#include <stdarg.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dlfcn.h>
#include <unistd.h>
#include <errno.h>
#include <glib.h>

/* This should come from dlfcn.h, not sure why it fails */
#define RTLD_NEXT ((void *) -1l)

int (* real__fopen) (const char *, const char *);

FILE * fopen (const char * filename, const char * mode) {
  int retval;
  const char * errval;
  char * realpath;

  if (!real__fopen) {
    real__fopen = dlsym (RTLD_NEXT, "fopen");
    if ((errval = dlerror ()) != NULL) {
      fprintf (stderr, "dlsym(fopen): %s\n", errval);
      errno = EFAULT;
      return -1;
    }
  }

  realpath = g_strdup (filename);

  /* Determine the proper dialog[F].tlk to use here */
  if (g_str_has_suffix (filename, ".tlk")) {
    char * dirname;
    char * pathname;
    char ** fileparts;
    const gchar * const * syslangs;
    int i;

    g_free (realpath);
    realpath = NULL;

    syslangs = g_get_language_names ();

    dirname = g_path_get_dirname (filename);
    pathname = g_path_get_basename (filename);

    fileparts = g_strsplit (pathname, ".", 2);

    for (i = 0; syslangs[i] != NULL; i++) {
      realpath = g_strdup_printf ("%s/%s-%s.%s", dirname, fileparts[0],
				  syslangs[i], fileparts[1]);
      if (g_file_test (realpath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
	break;
      } else {
	g_free (realpath);
	realpath = NULL;
      }
    }
    g_strfreev (fileparts);

    g_free (pathname);
    g_free (dirname);

    if (!realpath)
      realpath = g_strdup (filename);
  }

  retval = (* real__fopen) (realpath, mode);

  g_free (realpath);

  return retval;
}
